@@ -28,9 +28,5 @@ void druid_watch_sensitivity_widget (GladeXML *xml, const char *name);
 /* Some useful predicates */
 gboolean tree_view_has_selection (GladeXML   *xml,
 				  const char *name);
-gboolean toggle_button_is_active (GladeXML   *xml,
-				  const char *name);
-gboolean entry_has_text          (GladeXML   *xml,
-				  const char *name);
 
 #endif
