@@ -80,13 +80,20 @@ static GdkPixbuf *
 get_printer_icon (GnomeCupsPrinter *printer)
 {
 	char *icon_name;
+	GList *ptr, *emblems, *emblem_pixbufs = NULL;
 	GdkPixbuf *pixbuf;
 
 	gnome_cups_printer_get_icon (printer,
 				     &icon_name,
-				     NULL);
+				     &emblems);
 	pixbuf = load_icon (icon_name);
 	g_free (icon_name);
+	for (ptr = emblems; ptr != NULL; ptr = ptr->next)
+		if (ptr->data) {
+			GdkPixbuf *emblem = load_icon (ptr->data);
+			if (emblem != NULL)
+				emblem_pixbufs = g_list_prepend (emblem_pixbufs, emblem);
+		}
 
 	return pixbuf;
 }
@@ -189,8 +196,9 @@ gnome_printer_list_get_command_sensitivities (GnomePrinterList *list,
 					      gboolean *pause,
 					      gboolean *resume,
 					      gboolean *make_default,
-					      gboolean *delete,
-					      gboolean *properties)
+					      gboolean *del,
+					      gboolean *properties,
+					      gboolean *status)
 {
 	GList *l;
 	GList *selection;
@@ -218,12 +226,15 @@ gnome_printer_list_get_command_sensitivities (GnomePrinterList *list,
 		if (make_default) {
 			*make_default = FALSE;
 		}
-		if (delete) {
-			*delete = FALSE;
+		if (del) {
+			*del = FALSE;
 		}
 		if (properties) {
 			*properties = FALSE;
 		}
+		if (status) {
+			*status = FALSE;
+		}
 		return;
 	}
 	
@@ -260,25 +271,29 @@ gnome_printer_list_get_command_sensitivities (GnomePrinterList *list,
 	}
 	
 	if (pause) {
-		*pause = !new_printer_selected && !all_paused;
+		*pause = !new_printer_selected && !all_paused && gnome_cups_can_admin ();
 	}
 	
 	if (resume) {
-		*resume = !new_printer_selected && !all_running;
+		*resume = !new_printer_selected && !all_running && gnome_cups_can_admin ();
 	}
 	
 	if (make_default) {
 		*make_default = !new_printer_selected && num_selected == 1 && !has_default;
 	}
 	
-	if (delete) {
-		*delete = !new_printer_selected && gnome_cups_can_admin ();
+	if (del) {
+		*del = !new_printer_selected && gnome_cups_can_admin ();
 	}
 	
 	if (properties) {
 		*properties = !new_printer_selected;
 	}
 
+	if (status) {
+		*status = !new_printer_selected;
+	}
+
 	g_list_free (selection);
 }
 
@@ -292,40 +307,45 @@ create_selection_menu (GnomePrinterList *list)
 	gboolean pause;
 	gboolean resume;
 	gboolean make_default;
-	gboolean delete;
+	gboolean del;
 	gboolean properties;
+	gboolean status;
 
 	gnome_printer_list_get_command_sensitivities (list,
-						      &activate,
-						      &pause,
-						      &resume,
-						      &make_default,
-						      &delete,
-						      &properties);
+		&activate, &pause, &resume, &make_default,
+		&del, &properties, &status);
 	
 	selection = egg_icon_list_get_selected (EGG_ICON_LIST (list));
 
 	menu = gtk_menu_new ();
 	gtk_widget_show (menu);
 
-	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_OPEN, NULL);
+	if (g_list_length (selection) == 1 && selection->data == list->details->add_item) {
+		g_list_free (selection);
+
+		item = gtk_image_menu_item_new_from_stock (GTK_STOCK_ADD, NULL);
+		gtk_widget_show (item);
+		g_signal_connect (item, "activate", 
+				  G_CALLBACK (open_activated_cb),
+				  list);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+		return menu;
+	}
+	g_list_free (selection);
+
+	item = gtk_menu_item_new_with_label (_("Jobs"));
 	gtk_widget_show (item);
 	g_signal_connect (item, "activate", 
 			  G_CALLBACK (open_activated_cb),
 			  list);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
-	if (g_list_length (selection) == 1 && selection->data == list->details->add_item) {
-		g_list_free (selection);
-		return menu;
-	}
-	g_list_free (selection);
-
 	item = gtk_separator_menu_item_new ();
 	gtk_widget_show (item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);	
 
-	item = gtk_menu_item_new_with_label (_("Pause"));
+	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_MEDIA_PAUSE, NULL);
 	gtk_widget_show (item);
 	gtk_widget_set_sensitive (item, pause);
 	g_signal_connect (item, "activate", 
@@ -353,9 +373,9 @@ create_selection_menu (GnomePrinterList *list)
 	gtk_widget_show (item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);	
 
-	item = gtk_menu_item_new_with_label (_("Delete"));
+	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_REMOVE, NULL);
 	gtk_widget_show (item);
-	gtk_widget_set_sensitive (item, delete);
+	gtk_widget_set_sensitive (item, del);
 	g_signal_connect (item, "activate", 
 			  G_CALLBACK (delete_activated_cb),
 			  list);
@@ -365,7 +385,7 @@ create_selection_menu (GnomePrinterList *list)
 	gtk_widget_show (item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);	
 
-	item = gtk_menu_item_new_with_label (_("Properties"));
+	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_PROPERTIES, NULL);
 	gtk_widget_show (item);
 	gtk_widget_set_sensitive (item, properties);
 	g_signal_connect (item, "activate", 
